#include <netcdf.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>


#define SUC  0     
#define DIF  1
#define ERR  2

#define HELP "\
Compares metadata for two NetCDF files. Stops after 1st difference, by default. \
Error messages are sent to STDERR. Exit code 0 is returned if equivalent, \
1 if different and 2 for a severe error. \n\
\n\
Usage: nccmpmeta [OPTION...] file1 file2\
\n\
  -f             Forcefully compare, do not stop at 1st difference\n\
  -g             Compare global attributes\n\
  -h             Compare global history attribute\n\
                 (-g required)\n\
  -u             Prints these usage instructions\n\
  -v var1[,...]  Compare variable(s) var1[,...] only\n\
\n\
DOC / NOAA / GFDL\n\
\n\
Report bugs to first dot last @ noaa.gov | [first,last] = [Remik,Ziemlinski].\n\
" 

/***************************************
        nccmpmeta
*****************************************/
int nccmpmeta(  const char* file1, const char* file2, int force,
                int global, int history, char** variables, int nvars);
/* returns NC_EINVAL or NC_NOERR */
int cmpattval(  int nc1, int nc2, int varid1, int varid2, 
                char* name, int len, nc_type type);
void type2string(int type, char* str);
void printhelp();            
int check(int error, const char* errmsg);
int check1(int error, const char* errmsg, const char* str);
int check2(int error, const char* errmsg, const char* str1, const char* str2);
void freevarlist(char** list, int nvars);
char** getlistofvars(char *optarg, int* nvars);
int inStringList(char** list, char* str, int len);
void addvartolist(char** list, char* varname, int* nvars);

/*************************************** 
        getopt
*****************************************/
int getopt(int argc, char * const argv[], const char *optstring);
extern char *optarg;
extern int optind, opterr, optopt;














void printhelp()
{
        fprintf(stderr, HELP);
}
int check(int error, const char* errmsg)
{
        if(error)        
                fprintf(stderr, errmsg);

        return error;
}
int check1(int error, const char* errmsg, const char* str)
{
        if(error)        
                fprintf(stderr, errmsg, str);

        return error;
}
int check2(int error, const char* errmsg, const char* str1, const char* str2)
{
        if(error)        
                fprintf(stderr, errmsg, str1, str2);

        return error;
}
int nccmpmeta(  const char* file1, const char* file2, int force,
                int global, int history, char** variables, int nvars)
{        
        int nc1;
        int nc2;
        int ndims1;
        int ndims2;
        int nvars1;
        int nvars2;
        int natts1;
        int natts2;
        int recid1;
        int recid2;
        int dimid1;
        int dimid2;
        int attid1;
        int attid2;
        int varid1;
        int varid2;
        char name1[NC_MAX_NAME];
        char name2[NC_MAX_NAME];
        char recname1[NC_MAX_NAME];
        char recname2[NC_MAX_NAME];
        char varname1[NC_MAX_NAME];
        char varname2[NC_MAX_NAME];  
        char typestr1[32];
        char typestr2[32];      
        size_t len1;
        size_t len2;
        nc_type type1;
        nc_type type2;
        int dimids1[NC_MAX_DIMS];
        int dimids2[NC_MAX_DIMS];
        int nattsnoh1;
        int nattsnoh2;        
        int status; 
        int ncstatus;       
        int i;
        int j;
        int varlistlocallyalloc;
        int tmp1;
        int tmp2;
        
        status = SUC;
        
        if ((file1 == NULL) || (file2 == NULL))
                return ERR;
                
        ncstatus = nc_open(file1, NC_NOWRITE, &nc1);
        if (check1( ncstatus != NC_NOERR, "Failed to open %s\n", file1) )
        {
                return ERR;
        }

        ncstatus = nc_open(file2, NC_NOWRITE, &nc2);
        if (check1( ncstatus != NC_NOERR, "Failed to open %s\n", file2) )
        {
                return ERR;
        }
         
        ncstatus = nc_inq(nc1, &ndims1, &nvars1, &natts1, &recid1); 
        if (check1( ncstatus != NC_NOERR, "Failed to query %s\n", file1))
                return ERR;

        ncstatus = nc_inq(nc2, &ndims2, &nvars2, &natts2, &recid2); 
        if (check1( ncstatus != NC_NOERR, "Failed to query %s\n", file2))
                return ERR;
                
        /* get record names, used when comparing variable dimensions */
        if (recid1 != -1) 
        {
                ncstatus = nc_inq_dimname(nc1, recid1, recname1);
                if(ncstatus != NC_NOERR)
                {
                        status = ERR;  
                        fprintf(stderr, "Failed to query record name for %s\n", file1);                                       
                        if(!force) goto recover;
                }                       
        }        
        if (recid2 != -1) 
        {
                ncstatus = nc_inq_dimname(nc2, recid2, recname2);
                if(ncstatus != NC_NOERR)
                {
                        status = ERR;  
                        fprintf(stderr, "Failed to query record name for %s\n", file2);                                       
                        if(!force) goto recover;
                }                        
        }        








        if (nvars == 0)
        {
                /* we're comparing everything since no vars were supplied */
                if (ndims1 != ndims2)
                {
                        fprintf(stderr, "DIFFER : NUMBER : DIMENSIONS : %d : %d\n", ndims1, ndims2 );
                        if (force) status = DIF; else return DIF;
                }
                if (nvars1 != nvars2)
                {
                        fprintf(stderr, "DIFFER : NUMBER : VARIABLES : %d : %d\n", nvars1, nvars2 );
                        if (force) status = DIF; else return DIF;
                }

                if ((recid1 != -1) && (recid2 == -1))
                {
                        fprintf(stderr,"DIFFER : NUMBER : RECORDS : 1 : 0\n");
                        if (force) status = DIF; else return DIF;
                }
                if ((recid1 == -1) && (recid2 != -1))
                {
                        fprintf(stderr,"DIFFER : NUMBER : RECORDS : 0 : 1\n");
                        if (force) status = DIF; else return DIF;
                }
        }
        

















                                
        /* global attributes */
        if (global)
        {
                /* find # of global attributes, with or without "history" */
                if ( history == 0 ) 
                {
                        nattsnoh1 = natts1;
                        nattsnoh2 = natts2;

                        for(i = 0; i < natts1; ++i)
                        {
                                attid1 = i;
                                ncstatus = nc_inq_attname(nc1, NC_GLOBAL, attid1, name1);
                                if(check1(ncstatus != NC_NOERR, "Query failed on global attribute for file %s\n", file1))
                                        if(force) status = ERR; else return ERR;

                                if (strcmp(name1, "history") == 0)
                                {
                                        --nattsnoh1;
                                        break;
                                }
                        }                        
                        for(i = 0; i < natts2; ++i)
                        {
                                attid2 = i;
                                ncstatus = nc_inq_attname(nc2, NC_GLOBAL, attid2, name2);
                                if(check1(ncstatus != NC_NOERR, "Query failed on global attribute for file %s\n", file2))
                                        if(force) status = ERR; else return ERR;

                                if (strcmp(name2, "history") == 0)
                                {
                                        --nattsnoh2;
                                        break;
                                }
                        }                        
                        if(nattsnoh1 != nattsnoh2)
                        {
                                fprintf(stderr, "DIFFER : NUMBER : GLOBAL ATTRIBUTES EXCLUDING HISTORY : %d : %d\n", nattsnoh1, nattsnoh2);
                                if(force) status = DIF; else return DIF;
                        }
                } else     
                {                        
                        if(natts1 != natts2)
                        {
                                fprintf(stderr, "DIFFER : NUMBER : GLOBAL ATTRIBUTES : %d : %d\n", natts1, natts2);
                                if(force) status = DIF; else return DIF;
                        }
                }

                for(i = 0; i < natts1; ++i)
                {
                        attid1 = i;
                        ncstatus = nc_inq_attname(nc1, NC_GLOBAL, attid1, name1);
                        if (ncstatus != NC_NOERR)
                        {
                                fprintf(stderr, "Query failed for global attribute name in %s\n", file1);                        
                                status = ERR;
                                if(force) continue; else return ERR;
                        }
                        
                        if (
                                ( history != 0 ) || 
                                (strcmp(name1, "history") != 0)
                        )
                        {

                                ncstatus = nc_inq_att(nc1, NC_GLOBAL, name1, &type1, &len1);
                                if (check1(ncstatus != NC_NOERR, "Query failed on global attribute in %s\n", file1))
                                {
                                        status = ERR;
                                        if(force) continue; else return ERR;
                                }
                                
                                ncstatus = nc_inq_att(nc2, NC_GLOBAL, name1, &type2, &len2);
                                if (ncstatus != NC_NOERR)
                                {
                                        fprintf(stderr, "DIFFER : NAME : GLOBAL ATTRIBUTE : %s : DOES GLOBAL ATTRIBUTE EXIST IN %s ?\n", name1, file2);
                                        status = DIF;
                                        if(force) continue; else return DIF;
                                }

                                if (type1 != type2)
                                {
                                        type2string(type1, typestr1);
                                        type2string(type2, typestr2);
                                        fprintf(stderr, "DIFFER : TYPES : GLOBAL ATTRIBUTE : %s : %s : %s\n", name1, typestr1, typestr2);
                                        status = DIF;
                                        if(force) continue; else return DIF;
                                }
                                if (len1 != len2)
                                {
                                        fprintf(stderr, "DIFFER : LENGTHS : GLOBAL ATTRIBUTE : %s : %lu : %lu\n", name1, 
                                                        (unsigned long)len1, (unsigned long)len2);
                                        status = DIF;
                                        if(force) continue; else return DIF;
                                }
                                if (cmpattval(nc1,nc2,NC_GLOBAL,NC_GLOBAL,name1,len1,type1) != NC_NOERR)
                                {
                                        fprintf(stderr, "DIFFER : VALUES : GLOBAL ATTRIBUTE : %s\n", name1);
                                        status = DIF;
                                        if(force) continue; else return DIF;
                                }
                        }
                }
        }        
       
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        /* dimensions */
        /* compare only if no specific vars were given */
        if (nvars == 0)
        {        
                for(i = 0; i < ndims1; ++i)
                {
                        dimid1 = i;
                        ncstatus = nc_inq_dim(nc1, dimid1, name1, &len1);
                        if (check1(ncstatus != NC_NOERR, "Failed to query dimension in file %s\n", file1))
                        {
                                status = ERR;
                                if(force) continue; else return ERR;
                        }
                        ncstatus = nc_inq_dimid(nc2, name1, &dimid2);
                        if (ncstatus != NC_NOERR)
                        {
                                fprintf(stderr, "DIFFER : NAME : DIMENSION : %s : DOES DIMENSION EXIST IN %s ?\n", name1, file2);
                                status = DIF;
                                if(force) continue; else return DIF;
                        }
                        ncstatus = nc_inq_dim(nc2, dimid2, name2, &len2);
                        if (check2(ncstatus != NC_NOERR, "Failed to query dimension %s in file %s\n", name1, file2))
                        {
                                status = ERR;
                                if(force) continue; else return ERR;
                        }

                        if (len1 != len2)
                        {
                                fprintf(stderr, "DIFFER : LENGTHS : DIMENSION : %s : %lu : %lu\n", name1, 
                                                (unsigned long)len1, (unsigned long)len2);
                                status = DIF;
                                if(force) continue; else return DIF;
                        }
                }
        }        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        /* variables */    
        if (nvars == 0)
        {
                varlistlocallyalloc = 1;
                variables = (char **) malloc(NC_MAX_VARS * sizeof(char*));                
                /* make list of variables from both files */
                for(i = 0; i < nvars1; ++i)
                {
                        varid1 = i;
                        ncstatus = nc_inq_varname(nc1, varid1, varname1);
                        if(check1(ncstatus != NC_NOERR, "Failed to query variable in file %s\n", file1))
                        {
                                status = ERR;
                                if(force) 
                                {
                                        continue; 
                                } else 
                                {
                                        goto recover;
                                }
                        }        
                        addvartolist(variables, varname1, &nvars);
                }

                for(i = 0; i < nvars2; ++i)
                {
                        varid2 = i;
                        ncstatus = nc_inq_varname(nc2, varid2, varname2);
                        if(check1(ncstatus != NC_NOERR, "Failed to query variable in file %s\n", file2))
                        {
                                status = ERR; 
                                if(force) continue; else goto recover;
                        }
                               
                        if (inStringList(variables, varname2, nvars) == 0)
                                addvartolist(variables, varname2, &nvars);
                }                              
        } else
                varlistlocallyalloc = 0;        
           
        for(i = 0; i < nvars; ++i)
        {
                ncstatus = nc_inq_varid(nc1, variables[i], &varid1);
                if(ncstatus != NC_NOERR)
                {
                        fprintf(stderr, "DIFFER : NAME : VARIABLE : %s : DOES VARIABLE EXIST IN %s ?\n", 
                                        variables[i], file1);                                        
                        status = DIF;                                         
                        if(force) continue; else goto recover;
                }

                ncstatus = nc_inq_var(nc1, varid1, varname1, &type1, &ndims1, dimids1, &natts1);
                if(check1(ncstatus != NC_NOERR, "Failed to query variable in file %s\n", file1))
                {
                        status = ERR;                                         
                        if(force) continue; else goto recover;
                }
                
                ncstatus = nc_inq_varid(nc2, varname1, &varid2);
                if(ncstatus != NC_NOERR)
                {
                        fprintf(stderr, "DIFFER : NAME : VARIABLE : %s : DOES VARIABLE EXIST IN %s ?\n", 
                                        varname1, file2);                                        
                        status = DIF;                                         
                        if(force) continue; else goto recover;
                }
                
                ncstatus = nc_inq_var(nc2, varid2, varname2, &type2, &ndims2, dimids2, &natts2);
                if(check2(ncstatus != NC_NOERR, "Failed to query variable %s in file %s\n", varname1, file2))
                {
                        status = ERR;                                         
                        if(force) continue; else goto recover;
                }
                
                if(type1 != type2)
                {
                        type2string(type1, typestr1);
                        type2string(type2, typestr2);                        
                        fprintf(stderr, "DIFFER : TYPES : VARIABLE : %s : %s : %s\n", varname1, typestr1, typestr2);
                        status = DIF;                                         
                        if(force) continue; else goto recover;
                }
                if(ndims1 != ndims2)
                {
                        fprintf(stderr, "DIFFER : NUMBER : DIMENSIONS : VARIABLE : %s : %d : %d\n", varname1, ndims1, ndims2);
                        status = DIF;                                         
                        if(force) continue; else goto recover;
                }
                if(natts1 != natts2)
                {
                        fprintf(stderr, "DIFFER : NUMBER : ATTRIBUTES : VARIABLE : %s : %d : %d\n", varname1, natts1, natts2);
                        status = DIF;                                         
                        if(force) continue; else goto recover;
                }
                        
                /* dimensions */
                for(j = 0; j < ndims1; ++j)
                {
                        dimid1 = dimids1[j];
                        ncstatus = nc_inq_dim(nc1, dimid1, name1, &len1);
                        if (check2(ncstatus != NC_NOERR, "Failed to query variable %s dimension in file %s\n", varname1, file1))
                        {
                                status = ERR;                                         
                                if(force) continue; else goto recover;
                        }
                        dimid2 = dimids2[j];
                        ncstatus = nc_inq_dim(nc2, dimid2, name2, &len2);
                        if (check2(ncstatus != NC_NOERR, "Failed to query variable %s dimension in file %s\n", varname1, file1))
                        {
                                status = ERR;                                         
                                if(force) continue; else goto recover;
                        }
                        
                        if (strcmp(name1, name2) != 0)
                        {
                                fprintf(stderr, "DIFFER : VARIABLE : %s : DIMENSION : %d : %s : %s\n", 
                                                varname1, (j+1), name1, name2);
                                status = DIF;                                         
                                if(force) continue; else goto recover;
                        } 
                        
                        tmp1 = strcmp(name1, recname1);
                        tmp2 = strcmp(name2, recname2);
                        if ( (tmp1 == 0) && (tmp2 != 0) ) 
                        {
                                fprintf(stderr, "DIFFER : VARIABLE : %s : RECORD DIMENSION : %s : YES : NO\n", varname1, name1);
                                status = DIF;                                         
                                if(force) continue; else goto recover;                                
                        } else if ( (tmp1 != 0) && (tmp2 == 0) )  
                        {
                                fprintf(stderr, "DIFFER : VARIABLE : %s : RECORD DIMENSION : %s : NO : YES\n", varname1, name1);
                                status = DIF;                                         
                                if(force) continue; else goto recover;                                
                        }
                }

                /* attributes */
                for(j = 0; j < natts1; ++j)
                {
                        attid1 = j;
                        ncstatus = nc_inq_attname(nc1, varid1, attid1, name1);
                        if (check2(ncstatus != NC_NOERR, "Failed to query variable %s attribute in file %s\n", varname1, file1))
                        {
                                status = ERR;                                         
                                if(force) continue; else goto recover;
                        }
                            
                        ncstatus = nc_inq_att(nc1, varid1, name1, &type1, &len1);
                        if (check2(ncstatus != NC_NOERR, "Failed to query variable %s attribute in file %s\n", varname1, file1))
                        {
                                status = ERR;                                         
                                if(force) continue; else goto recover;
                        }

                        ncstatus = nc_inq_att(nc2, varid2, name1, &type2, &len2);
                        if (ncstatus != NC_NOERR)
                        {
                                fprintf(stderr, "DIFFER : NAME : ATTRIBUTE : %s : VARIABLE : %s : DOES ATTRIBUTE EXIST IN %s ?\n", 
                                                name1, varname1, file2);
                                status = DIF;                                         
                                if(force) continue; else goto recover;
                        }

                        if (type1 != type2)
                        {
                                type2string(type1, typestr1);
                                type2string(type2, typestr2);                                                
                                fprintf(stderr, "DIFFER : TYPES : ATTRIBUTE : %s : VARIABLE : %s : %s : %s\n", 
                                                name1, varname1, typestr1, typestr2);
                                status = DIF;                                         
                                if(force) continue; else goto recover;
                        }
                        if (len1 != len2)
                        {
                                fprintf(stderr, "DIFFER : LENGTHS : ATTRIBUTE : %s : VARIABLE : %s : %lu : %lu\n", 
                                                name1, varname1, (unsigned long)len1, (unsigned long)len2);
                                status = DIF;                                         
                                if(force) continue; else goto recover;
                        }
                        if (cmpattval(nc1,nc2,varid1,varid2,name1,len1,type1) != NC_NOERR)
                        {
                                fprintf(stderr, "DIFFER : VALUES : ATTRIBUTE : %s : VARIABLE : %s\n", 
                                                name1, varname1);
                                status = DIF;                                         
                                if(force) continue; else goto recover;
                        }
                }
        }

        
recover:
        if (varlistlocallyalloc)
                freevarlist(variables, nvars);
       
        nc_close(nc1);
        nc_close(nc2);
               
        return status;
}
int cmpattval(int nc1, int nc2, int varid1, int varid2, char* name, int len, nc_type type)
{
        char* c1;
        unsigned char* uc1;
        short* s1;
        int* i1;
        float* f1;
        double* d1;
        char* c2;
        unsigned char* uc2;
        short* s2;
        int* i2;
        float* f2;
        double* d2;
        int status;
        int i;
        
        if (name == NULL) return NC_EINVAL;
        
        switch(type)
        {
        case NC_BYTE:
                uc1 = malloc(sizeof(unsigned char)*len);
                uc2 = malloc(sizeof(unsigned char)*len);                
                status = nc_get_att_uchar(nc1, varid1, name, uc1);
                if (status != NC_NOERR)
                {
                        free(uc1); free(uc2); return NC_EINVAL;
                }
                status = nc_get_att_uchar(nc2, varid2, name, uc2);
                if (status != NC_NOERR)
                {
                        free(uc1); free(uc2); return NC_EINVAL;
                }
                if (strncmp((const char*)uc1,(const char*)uc2,len) != 0)
                {
                        free(uc1); free(uc2); return NC_EINVAL;
                }
               free(uc1); 
               free(uc2); 
        break;
        case NC_CHAR:
                c1 = malloc(sizeof(char)*len);
                c2 = malloc(sizeof(char)*len);                
                status = nc_get_att_text(nc1, varid1, name, c1);
                if (status != NC_NOERR)
                {
                        free(c1); free(c2); return NC_EINVAL;
                }
                status = nc_get_att_text(nc2, varid2, name, c2);
                if (status != NC_NOERR)
                {
                        free(c1); free(c2); return NC_EINVAL;
                }
                if (strncmp(c1,c2,len) != 0)
                {
                        free(c1); free(c2); return NC_EINVAL;
                }
               free(c1); 
               free(c2); 
        break;
        case NC_SHORT:
                s1 = malloc(sizeof(short)*len);
                s2 = malloc(sizeof(short)*len);                
                status = nc_get_att_short(nc1, varid1, name, s1);
                if (status != NC_NOERR)
                {
                        free(s1); free(s2); return NC_EINVAL;
                }
                status = nc_get_att_short(nc2, varid2, name, s2);
                if (status != NC_NOERR)
                {
                        free(s1); free(s2); return NC_EINVAL;
                }
                for(i = 0; i < len; ++i)
                {
                        if (s1[i] != s2[i])
                        {
                                free(s1); free(s2); return NC_EINVAL;
                        }
                }
               free(s1); 
               free(s2); 
        break;        
        case NC_INT:
                i1 = malloc(sizeof(int)*len);
                i2 = malloc(sizeof(int)*len);                
                status = nc_get_att_int(nc1, varid1, name, i1);
                if (status != NC_NOERR)
                {
                        free(i1); free(i2); return NC_EINVAL;
                }
                status = nc_get_att_int(nc2, varid2, name, i2);
                if (status != NC_NOERR)
                {
                        free(i1); free(i2); return NC_EINVAL;
                }
                for(i = 0; i < len; ++i)
                {
                        if (i1[i] != i2[i])
                        {
                                free(i1); free(i2); return NC_EINVAL;
                        }
                }
               free(i1); 
               free(i2); 
        break;        
        case NC_FLOAT:
                f1 = malloc(sizeof(float)*len);
                f2 = malloc(sizeof(float)*len);                
                status = nc_get_att_float(nc1, varid1, name, f1);
                if (status != NC_NOERR)
                {
                        free(f1); free(f2); return NC_EINVAL;
                }
                status = nc_get_att_float(nc2, varid2, name, f2);
                if (status != NC_NOERR)
                {
                        free(f1); free(f2); return NC_EINVAL;
                }
                for(i = 0; i < len; ++i)
                {
                        if (f1[i] != f2[i])
                        {
                                free(f1); free(f2); return NC_EINVAL;
                        }
                }
               free(f1); 
               free(f2); 
        break;        
        case NC_DOUBLE:
                d1 = malloc(sizeof(double)*len);
                d2 = malloc(sizeof(double)*len);                
                status = nc_get_att_double(nc1, varid1, name, d1);
                if (status != NC_NOERR)
                {
                        free(d1); free(d2); return NC_EINVAL;
                }
                status = nc_get_att_double(nc2, varid2, name, d2);
                if (status != NC_NOERR)
                {
                        free(d1); free(d2); return NC_EINVAL;
                }
                for(i = 0; i < len; ++i)
                {
                        if (d1[i] != d2[i])
                        {
                                free(d1); free(d2); return NC_EINVAL;
                        }
                }
               free(d1); 
               free(d2); 
        break;        
        }
        
        return NC_NOERR;
}
void type2string(int type, char* str)
{
        switch(type)
        {
        case NC_BYTE:
        strcpy(str, "BYTE");
        break;
        case NC_CHAR:
        strcpy(str, "CHAR");
        break;
        case NC_SHORT:      
        strcpy(str, "SHORT");
        break;        
        case NC_INT:  
        strcpy(str, "INT");  
        break;        
        case NC_FLOAT:     
        strcpy(str, "FLOAT");
        break;        
        case NC_DOUBLE:
        strcpy(str, "DOUBLE");
        break;
        default:
        break;
        }                 
}
char** getlistofvars(char *optarg, int* nvars)
{
        char *p;
        char **list;
        char **plist;
        
        p = optarg;
        *nvars = 1;
        
        /* find # comma-delimited vars */
        while (*p++) { if (*p == ',') ++(*nvars); }

        list = (char **) malloc(*nvars * sizeof(char*));
        
        if (list == NULL) 
        {
                return NULL;
        }

        plist = list;

        /* copy var names into list */
        for (p = (char*)strtok(optarg, ",");
             p != NULL;
             p = (char*)strtok((char*) NULL, ",")) 
        {
                *plist = (char*) malloc(strlen(p) + 1);
                
                if (*plist == NULL) 
                {
                        freevarlist(list, *nvars);
                        list = NULL;
                        break;
                }
        
                strcpy(*plist, p);
                plist++;
        }

        return list;
}
void freevarlist(char** list, int nvars)
{
        for(; nvars > 0; --nvars)
        {
                free(list[nvars-1]);
        }
        
        free(list);
}
/* 0 - false, 1 - true */
int inStringList(char** list, char* str, int len)
{       
        if(list == NULL) return 0;
        
        for(; len > 0; --len)
        {
                if(strcmp(list[len-1],str) == 0)
                        return 1;        
        }
        return 0;
}
void addvartolist(char** list, char* varname, int* nvars)
{
        if ((list == NULL) || (varname == NULL)) return;                       
        
        list[*nvars] = (char*) malloc(strlen(varname) + 1);
        strcpy( list[*nvars], varname);
        ++(*nvars);
}


















int main(int argc, char* argv[])
{
        int status;
        int c;
        int errflag;
        int nvars;
        /* options */
        int force;
        int global;
        int history;
        char** variables;
        
        nvars = force = global = history = errflag = c = 0;
        status = ERR;
        variables = NULL;
        
        /* parse options */
        while ((c = getopt(argc, argv, ":fghuv:")) != -1) {
            switch(c) {
            case 'f':
                force = 1;
                break;
            case 'g':
                global = 1;
                break;
            case 'h':
                history = 1;
                break;
            case 'u':
                errflag = 1;
                break;
            case 'v':
                variables = getlistofvars(optarg, &nvars);
                break;
            case ':':
                fprintf(stderr, "-%c without argument\n\n", optopt);
                errflag = 1;
                break;
            case '?':
                fprintf(stderr, "Unknown argument %c\n\n", optopt);
                errflag = 1;
                break;
            }
        }

        argc -= optind;
        argv += optind;
        if (    (argc != 2) ||
                (argv[0] == NULL) ||
                (argv[1] == NULL) 
           )
                errflag = 1;

        if (history && !global)
        {
                fprintf(stderr, "-g required for -h\n");
                errflag = 1;
        }
                
        if (errflag) 
        {
                printhelp();
                status = ERR;
        } else
        {
                status = nccmpmeta(    argv[0], argv[1], 
                                        force, global, history, 
                                        variables, nvars);        
        }    
        
        freevarlist(variables, nvars);
                         
        return status;
}  
/*======================================================================
  U.S. Department of Commerce (DOC) Software License for script 
  developed at the Geophysical Fluid Dynamics Laboratory/NOAA.

  1. Scope of License

   Subject to all the terms and conditions of this license, DOC grants
   USER the royalty-free, nonexclusive, nontransferable, and worldwide
   rights to reproduce, modify, and distribute this script
   developed at the Geophysical Fluid Dynamics Laboratory/NOAA, herein
   referred to as the Product.

  2. Conditions and Limitations of Use

   Warranties.  Neither the U.S. Government, nor any agency or
     employee thereof, makes any warranties, expressed or implied,
     with respect to the Product provided under this License,
     including but not limited to the implied warranties or
     merchantability and fitness for any particular purpose.

   Liability.  In no event shall the U.S. Government, nor any agency
     or employee thereof, be liable for any  direct, indirect, or
     consequential damages flowing from the use of the Product
     provided under this License.

   Non-Assignment.  Neither this License nor any rights granted
     hereunder are transferable or assignable without the explicit
     prior written consent of DOC. 

   Names and Logos.  USER shall not substitute its name or logo for the
     name or logo of DOC, or any of its agencies, in identification of
     the Product.

   Export of technology.  USER shall comply with all U.S. laws and
     regulations restricting the export of the Product to other
     countries.  

   Governing Law.  This License shall be governed by the laws of 
     United States as interpreted and applied by the Federal courts 
     in the District of Columbia.

  3. Term of License This License shall remain in effect as long as USER
     uses the Product in accordance with Paragraphs 1 and 2.
======================================================================*/
